==> w1druppel-0.1.7.tgz: package/src/ds2482.c <==
#include <math.h>
#include <stdbool.h>
#include <stdint.h>
#include <unistd.h>

#include <alleycat.h>

#include "private/benchmark.h"
#include "private/ds2482.h"
#include "private/i2c.h"
#include "private/search.h"
#include "private/util.h"

// --- @todo remove
#define IDLE_EXPERIMENT

/* Select the given bus and prepare for search.
 * Returns whether a device was detected in &presence.
 */
bool ds2482_bus_search_init (struct master * master, const struct bus * const bus, bool * const presence) {
  try (ds2482_set_overdrive, (master, false))
  try (ds2482_bus_select, (master, bus))
  try (ds2482_w1_reset_check_presence, (master, presence))
  try (ds2482_wait_idle, (master, NULL))
  return true;
}

/* Search for exactly 1 device.
 * `ds2482_bus_search_init` must be called right before this.
 */
bool ds2482_bus_search_start (struct master * master) {
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd_with_data, (master, CMD_W1_WRITE_BYTE, W1_SEARCH_ALL))
  return true;
}

// --- @todo stub, always returns true for now, channel hardcoded.
// --- selected_bus gets set on failure as well (just like w1direct)
bool ds2482_bus_select (struct master * const master, const struct bus * const bus) {
  uint8_t bus_number = bus->number;
  if (master->selected_bus == bus_number) return true;
  // --- @todo is this important?
  master->selected_bus = bus_number;
  try_with_info (i2c_cmd_with_data, (master, CMD_CHANNEL_SELECT, bus->chan_select), ac_warn_perr, "CMD_CHANNEL_SELECT")
  uint8_t ack = 0;
  try (i2c_read_byte, (master, &ack))
  if (ack != bus->chan_select_ack) {
    ac_warn ("Channel select command succeeded but channel not selected");
    return false;
  }
  return true;
}

// --- note: "The overdrive speed is intended for use only on very short
// connections and is never suitable for use in 1-Wired networks."
static bool set_overdrive_on_master (const struct master * const master, bool overdrive) {
  try (ds2482_wait_idle, (master, NULL))

  uint8_t val_in = overdrive ? CFG_REG_1WS : 0;
  // --- @todo using strong pullup during scratchpad copying and/or SHA1
  // calculation helps deliver more power and might make it more reliable.
  // It turns off and sets itself to 0 by issuing a command, issuing reset,
  // or manually setting to 0. However, issuing reset while it's enabled is
  // a bad idea because it can blow out the device and/or mess up the
  // presence detection.
  // uint8_t val_in = overdrive ? (CFG_REG_1WS | CFG_REG_SPU) : 0;
  uint8_t val = calculate_config (val_in);

  try_info (i2c_cmd_with_data, (master, CMD_WRITE_CONFIG, val), "CMD_WRITE_CONFIG")
  return true;
}

static bool set_overdrive_on_devices (const struct master * const master, bool overdrive) {
  try (ds2482_w1_reset, (master))
  if (overdrive) try (ds2482_write_byte, (master, W1_SKIP_ROM_OVERDRIVE))
  return true;
}

bool ds2482_set_overdrive (const struct master * const master, bool const overdrive) {
  // --- master set must occur immediately after device set.
  try (set_overdrive_on_devices, (master, overdrive))
  try (set_overdrive_on_master, (master, overdrive))
  return true;
}

bool ds2482_wait_idle (const struct master * const master, uint8_t *status) {
  // benchmark ("wait_idle start");
  uint8_t val;

  try_info (ds2482_select_register, (master, SELECT_CODE_STATUS), "SELECT_CODE_STATUS")
  do try (i2c_read_byte, (master, &val))
  while (val & STATUS_BUSY);

  if (status != NULL) *status = val;
  // benchmark ("wait_idle end");
  return true;
}

bool ds2482_select_register (const struct master * const master, uint8_t reg) {
  try_info (i2c_cmd_with_data, (master, CMD_SET_READ_PTR, reg), "CMD_SET_READ_PTR")
  return true;
}

bool ds2482_triplet (const struct master * const master, uint8_t dir, uint8_t *triplet) {
  uint8_t status;

// #ifdef IDLE_EXPERIMENT
// usleep (1000);
// #else
  try (ds2482_wait_idle, (master, NULL))
// #endif

  // --- @todo in the example it's 0x80
  try_info (i2c_cmd_with_data, (master, CMD_W1_TRIPLET, dir ? 0xFF : 0), "CMD_W1_TRIPLET")

#ifdef IDLE_EXPERIMENT
usleep (100);
try_info (ds2482_select_register, (master, SELECT_CODE_STATUS), "SELECT_CODE_STATUS")
try (i2c_read_byte, (master, &status))
#else
  try (ds2482_wait_idle, (master, &status))
#endif

  *triplet = status >> 5;
  return true;
}

static bool w1_reset (const struct master * const master, bool * const ret) {
  uint8_t read;
  try (ds2482_wait_idle, (master, NULL))
  try_info (i2c_cmd, (master, CMD_W1_RESET), "CMD_W1_RESET")
  try (ds2482_wait_idle, (master, &read))
  if (ret != NULL) *ret = read & STATUS_PRESENCE_PULSE_DETECT;
  return true;
}

/* Sends CMD_W1_RESET to devices on the selected bus and reads the
 * STATUS_PRESENCE_PULSE_DETECT bit.
 *
 * Not to be confused with CMD_RESET, which initialises the master, and is only used at the very
 * beginning and doesn't have its own function.
 */
bool ds2482_w1_reset (const struct master * const master) {
  return w1_reset (master, NULL);
}

/* Sends CMD_W1_RESET to devices on the selected bus and reads the
 * STATUS_PRESENCE_PULSE_DETECT bit. See `ds2482_w1_reset`.
 */
bool ds2482_w1_reset_check_presence (const struct master * const master, bool * const ret) {
  return w1_reset (master, ret);
}

// 20ms.
/* Init key command sequence on selected bus, for a particular key id. Use this if there are
 * multiple keys on the same bus.
 */
bool ds2482_key_command_init_match_rom (const struct master * const master, const struct key * const key) {
  benchmark ("key_command_init start");
  uint64_t device_id = key->device_id;
  try (ds2482_w1_reset, (master))
  try_info (ds2482_write_byte, (master, W1_MATCH_ROM), "W1_MATCH_ROM")
  for (int i = 0; i < 8; i++) {
    try_info (ds2482_write_byte, (master, (uint8_t) (device_id >> i*8)), "device bit")
  }
  benchmark ("key_command_init end");
  return true;
}

// 20ms.
/* Init key command sequence on selected bus, without specifyin the key id. Use this if there is
 * only a single key one each bus because it's faster than `ds2482_key_command_init_match_rom`.
 */
bool ds2482_key_command_init_skip_rom (const struct master * const master) {
  benchmark ("key_command_init start");
  try (ds2482_w1_reset, (master))
  try_info (ds2482_write_byte, (master, W1_SKIP_ROM), "W1_SKIP_ROM")
  benchmark ("key_command_init end");
  return true;
}

/* Write to the selected bus.
 */
bool ds2482_write_byte (const struct master * const master, uint8_t byte) {
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd_with_data, (master, CMD_W1_WRITE_BYTE, byte))
  // @todo idle here?
  return true;
}

/* Write to the selected bus.
 */
bool ds2482_write_bytes (const struct master * const master, uint8_t *bytes, uint8_t count) {
  for (int i = 0; i < count; i++)
    try (ds2482_write_byte, (master, bytes[i]));
  return true;
}

/* Read from the devices on the selected bus.
 */
bool ds2482_read_byte (const struct master * const master, uint8_t *read) {
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd, (master, CMD_W1_READ_BYTE))
  try (ds2482_wait_idle, (master, NULL))
  try (ds2482_select_register, (master, SELECT_CODE_DATA))
  try (i2c_read_byte, (master, read))
  return true;
}

static bool touch_bit (const struct master * const master, bool const bit, bool * const ret) {
  uint8_t status;
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd_with_data, (master, CMD_W1_SINGLE_BIT, bit ? 0x80 : 0))
  try (ds2482_wait_idle, (master, &status))
  if (ret != NULL) *ret = (status & STATUS_SINGLE_BIT_READ) != 0;
  return true;
}

/* Read bit from the selected bus.
 */
bool ds2482_read_bit (const struct master * const master, bool * const ret) {
  try (touch_bit, (master, 1, ret))
  return true;
}

/* Write bit to the selected bus.
 */
bool ds2482_write_bit (const struct master * const master, bool const bit) {
  try (touch_bit, (master, bit, NULL))
  return true;
}

/* Read from the selected bus.
 */
bool ds2482_read_bytes (const struct master * const master, uint8_t *read, uint8_t count) {
  for (int i = 0; i < count; i++)
    try (ds2482_read_byte, (master, read + i))
  return true;
}

int ds2482_num_buses (int ds2482_type) {
  if (ds2482_type == DS2482_TYPE_100) return 1;
  if (ds2482_type == DS2482_TYPE_800) return 8;
  return -1;
}

bool ds2482_read_overdrive_on_master (const struct master * const master, bool *overdrive __attribute__ ((unused))) {
  uint8_t read;
  try (ds2482_select_register, (master, SELECT_CODE_CONFIG))
  try (i2c_read_byte, (master, &read))
  *overdrive = (read & CFG_REG_1WS) >> (int) log2 (CFG_REG_1WS);
  return true;
}

// @todo wrong (it reads the master). not sure if this is possible.
bool ds2482_read_overdrive_on_devices (const struct master * const master, bool *overdrive __attribute__ ((unused))) {
  uint8_t read;
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd, (master, CMD_W1_READ_BYTE))
  try (ds2482_wait_idle, (master, NULL))
  try (ds2482_select_register, (master, SELECT_CODE_CONFIG))
  try (i2c_read_byte, (master, &read))
  printf("ds2482_read_overdrive_on_devices: %d\n", read);
  return true;
}

==> w1druppel-0.2.3.tgz: package/src/ds2482.c <==
#include <math.h>
#include <stdbool.h>
#include <stdint.h>
#include <unistd.h>

#include <alleycat.h>

#include "private/benchmark.h"
#include "private/ds2482.h"
#include "private/i2c.h"
#include "private/search.h"
#include "private/util.h"

// --- @todo remove
#define IDLE_EXPERIMENT

/* Select the given bus and prepare for search.
 * Returns whether a device was detected in &presence.
 */
bool ds2482_bus_search_init (struct master * master, const struct bus * const bus, uint8_t cfg, bool * const presence) {
  // --- we set overdrive to low here, or else it breaks the search.
  try (ds2482_write_configuration, (master, cfg & ~CFG_REG_1WS))
  try (ds2482_bus_select, (master, bus))
  try (ds2482_w1_reset_check_presence, (master, presence))
  try (ds2482_wait_idle, (master, NULL))
  return true;
}

/* Search for exactly 1 device.
 * `ds2482_bus_search_init` must be called right before this.
 */
bool ds2482_bus_search_start (struct master * master) {
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd_with_data, (master, CMD_W1_WRITE_BYTE, W1_SEARCH_ALL))
  return true;
}

// --- @todo stub, always returns true for now, channel hardcoded.
// --- selected_bus gets set on failure as well (just like w1direct)
bool ds2482_bus_select (struct master * const master, const struct bus * const bus) {
  uint8_t bus_number = bus->number;
  if (master->selected_bus == bus_number) return true;
  // --- @todo is this important?
  master->selected_bus = bus_number;
  try_with_info (i2c_cmd_with_data, (master, CMD_CHANNEL_SELECT, bus->chan_select), ac_warn_perr, "CMD_CHANNEL_SELECT")
  uint8_t ack = 0;
  try (i2c_read_byte, (master, &ack))
  if (ack != bus->chan_select_ack) {
    ac_warn ("Channel select command succeeded but channel not selected");
    return false;
  }
  return true;
}

// --- note: "The overdrive speed is intended for use only on very short
// connections and is never suitable for use in 1-Wired networks."
static bool configure_master (const struct master * const master, uint8_t cfg) {
  try (ds2482_wait_idle, (master, NULL))

  /* @todo using strong pullup during scratchpad copying and/or SHA1
   * calculation can help deliver more power and might make it more reliable.
   * It turns off and sets itself to 0 by issuing a command, issuing reset,
   * or manually setting to 0. However, issuing reset while it's enabled is
   * a bad idea because it can blow out the device and/or mess up the
   * presence detection.
   *   val_in |= CFG_REG_SPU;
   */

  uint8_t val = calculate_config (cfg);
  try_info (i2c_cmd_with_data, (master, CMD_WRITE_CONFIG, val), "CMD_WRITE_CONFIG")
  return true;
}

// --- need to be careful if there are multiple slaves on a bus (see DS1961S
// spec under 'Overdrive Skip ROM').
static bool set_overdrive_on_devices (const struct master * const master, bool overdrive) {
  try (ds2482_w1_reset, (master))
  if (overdrive) try (ds2482_write_byte, (master, W1_SKIP_ROM_OVERDRIVE))
  return true;
}

bool ds2482_write_configuration (const struct master * const master, uint8_t cfg) {
  // --- master set must occur immediately after device set.
  try (set_overdrive_on_devices, (master, get_bit (cfg, CFG_REG_1WS)))
  try (configure_master, (master, cfg))
  return true;
}

bool ds2482_wait_idle (const struct master * const master, uint8_t *status) {
  // benchmark ("wait_idle start");
  uint8_t val;

  try_info (ds2482_select_register, (master, SELECT_CODE_STATUS), "SELECT_CODE_STATUS")
  do try (i2c_read_byte, (master, &val))
  while (val & STATUS_BUSY);

  if (status != NULL) *status = val;
  // benchmark ("wait_idle end");
  return true;
}

bool ds2482_select_register (const struct master * const master, uint8_t reg) {
  try_info (i2c_cmd_with_data, (master, CMD_SET_READ_PTR, reg), "CMD_SET_READ_PTR")
  return true;
}

bool ds2482_triplet (const struct master * const master, uint8_t dir, uint8_t *triplet) {
  uint8_t status;

// #ifdef IDLE_EXPERIMENT
// usleep (1000);
// #else
  try (ds2482_wait_idle, (master, NULL))
// #endif

  // --- @todo in the example it's 0x80
  try_info (i2c_cmd_with_data, (master, CMD_W1_TRIPLET, dir ? 0xFF : 0), "CMD_W1_TRIPLET")

#ifdef IDLE_EXPERIMENT
usleep (100);
try_info (ds2482_select_register, (master, SELECT_CODE_STATUS), "SELECT_CODE_STATUS")
try (i2c_read_byte, (master, &status))
#else
  try (ds2482_wait_idle, (master, &status))
#endif

  *triplet = status >> 5;
  return true;
}

static bool w1_reset (const struct master * const master, bool * const ret) {
  uint8_t read;
  try (ds2482_wait_idle, (master, NULL))
  try_info (i2c_cmd, (master, CMD_W1_RESET), "CMD_W1_RESET")
  try (ds2482_wait_idle, (master, &read))

  /* We don't actually do anything with this: in principle it's always true
   * if there's presence. It means someone puts the key crooked on the edge
   * of the probe, short-circuiting the probe with ground.
   */
  bool is_short = read & STATUS_SHORT_DETECTED;
  (void) is_short;

  if (ret != NULL) *ret = read & STATUS_PRESENCE_PULSE_DETECT;
  return true;
}

/* Sends CMD_W1_RESET to devices on the selected bus and reads the
 * STATUS_PRESENCE_PULSE_DETECT bit.
 *
 * Not to be confused with CMD_RESET, which initialises the master, and is only used at the very
 * beginning and doesn't have its own function.
 */
bool ds2482_w1_reset (const struct master * const master) {
  return w1_reset (master, NULL);
}

/* Sends CMD_W1_RESET to devices on the selected bus and reads the
 * STATUS_PRESENCE_PULSE_DETECT bit. See `ds2482_w1_reset`.
 */
bool ds2482_w1_reset_check_presence (const struct master * const master, bool * const ret) {
  return w1_reset (master, ret);
}

// 20ms.
/* Init key command sequence on selected bus, for a particular key id. Use this if there are
 * multiple keys on the same bus.
 */
bool ds2482_key_command_init_match_rom (const struct master * const master, const struct key * const key) {
  benchmark ("key_command_init start");
  uint64_t device_id = key->device_id;
  try (ds2482_w1_reset, (master))
  try_info (ds2482_write_byte, (master, W1_MATCH_ROM), "W1_MATCH_ROM")
  for (int i = 0; i < 8; i++) {
    try_info (ds2482_write_byte, (master, (uint8_t) (device_id >> i*8)), "device bit")
  }
  benchmark ("key_command_init end");
  return true;
}

// 20ms.
/* Init key command sequence on selected bus, without specifyin the key id. Use this if there is
 * only a single key one each bus because it's faster than `ds2482_key_command_init_match_rom`.
 */
bool ds2482_key_command_init_skip_rom (const struct master * const master) {
  benchmark ("key_command_init start");
  try (ds2482_w1_reset, (master))
  try_info (ds2482_write_byte, (master, W1_SKIP_ROM), "W1_SKIP_ROM")
  benchmark ("key_command_init end");
  return true;
}

/* Write to the selected bus.
 */
bool ds2482_write_byte (const struct master * const master, uint8_t byte) {
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd_with_data, (master, CMD_W1_WRITE_BYTE, byte))
  try (ds2482_wait_idle, (master, NULL))
  return true;
}

/* Write to the selected bus.
 */
bool ds2482_write_bytes (const struct master * const master, uint8_t *bytes, uint8_t count) {
  for (int i = 0; i < count; i++)
    try (ds2482_write_byte, (master, bytes[i]));
  return true;
}

/* Read from the devices on the selected bus.
 */
bool ds2482_read_byte (const struct master * const master, uint8_t *read) {
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd, (master, CMD_W1_READ_BYTE))
  try (ds2482_wait_idle, (master, NULL))
  try (ds2482_select_register, (master, SELECT_CODE_DATA))
  try (i2c_read_byte, (master, read))
  return true;
}

static bool touch_bit (const struct master * const master, bool const bit, bool * const ret) {
  uint8_t status;
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd_with_data, (master, CMD_W1_SINGLE_BIT, bit ? 0x80 : 0))
  try (ds2482_wait_idle, (master, &status))
  if (ret != NULL) *ret = (status & STATUS_SINGLE_BIT_READ) != 0;
  return true;
}

/* Read bit from the selected bus.
 */
bool ds2482_read_bit (const struct master * const master, bool * const ret) {
  try (touch_bit, (master, 1, ret))
  return true;
}

/* Write bit to the selected bus.
 */
bool ds2482_write_bit (const struct master * const master, bool const bit) {
  try (touch_bit, (master, bit, NULL))
  return true;
}

/* Read from the selected bus.
 */
bool ds2482_read_bytes (const struct master * const master, uint8_t *read, uint8_t count) {
  for (int i = 0; i < count; i++)
    try (ds2482_read_byte, (master, read + i))
  return true;
}

int ds2482_num_buses (int ds2482_type) {
  if (ds2482_type == DS2482_TYPE_100) return 1;
  if (ds2482_type == DS2482_TYPE_800) return 8;
  return -1;
}

bool ds2482_read_overdrive_on_master (const struct master * const master, bool *overdrive __attribute__ ((unused))) {
  uint8_t read;
  try (ds2482_select_register, (master, SELECT_CODE_CONFIG))
  try (i2c_read_byte, (master, &read))
  *overdrive = (read & CFG_REG_1WS) >> (int) log2 (CFG_REG_1WS);
  return true;
}

// @todo wrong (it reads the master). not sure if this is possible.
bool ds2482_read_overdrive_on_devices (const struct master * const master, bool *overdrive __attribute__ ((unused))) {
  uint8_t read;
  try (ds2482_wait_idle, (master, NULL))
  try (i2c_cmd, (master, CMD_W1_READ_BYTE))
  try (ds2482_wait_idle, (master, NULL))
  try (ds2482_select_register, (master, SELECT_CODE_CONFIG))
  try (i2c_read_byte, (master, &read))
  printf("ds2482_read_overdrive_on_devices: %d\n", read);
  return true;
}
